/**
 *	File	: BListTest.cpp
 *	Author	: Dariusz "DKnoto" Knociński
 *	Project	: Haiku OS / Terminal
 *	Site	: ul.Gen.Zajączka 21/1, 01-505 Warszawa 
 *	Date	: 2012/12/05 12:20
 *	Notes	: Test properties of the BList class.
 */

#include <stdio.h>
#include <posix/strings.h>

#include <os/support/String.h>
#include <os/support/List.h>

static int CompareStrAsDecimalValue(const void *ptr1, const void *ptr2)
{
	const char *pStr1 = *(const char **)ptr1;
	const char *pStr2 = *(const char **)ptr2;
	const int len1 = strlen(pStr1);
	const int len2 = strlen(pStr2);
	
	int retval;
	
	if (len1 == len2)
		retval = strcmp(pStr1, pStr2);
	else
	if (len1 < len2)
		retval = -10;
	else
		retval = 10;
	
	printf( "CompareStrAsDecimalValue( \"%s\", \"%s\" ) == %d\n", pStr1, pStr2,
		retval);

	return retval;
}

int main(void)
{
	printf("BList test:\n");

	printf("  create default list...\n");
	BList *pList = new BList();
	printf("  p == %p\n", (void *)pList);

	pList->AddItem((void *)"8");
	pList->AddItem((void *)"10");
	pList->AddItem((void *)"12");
	pList->AddItem((void *)"14");
	pList->AddItem((void *)"16");
	pList->AddItem((void *)"18");

	printf("  pList->CountItems() == %ld\n", pList->CountItems());
	printf("  pList->FirstItem() == \"%s\"\n", (const char *)pList->FirstItem());
	printf("  pList->LastItem() == \"%s\"\n", (const char *)pList->LastItem());
	
	const char *strTrue = "True";
	const char *strFalse = "False";
	
	const char *testItem1 = "7";
	printf("  pList->HasItem(\"%s\") == \"%s\"\n", testItem1, pList->HasItem(testItem1) ? strTrue : strFalse);
	printf("  pList->IndexOf(\"%s\") == %ld\n", testItem1, pList->IndexOf(testItem1));
	printf("  pList->AddItem(\"%s\") == \"%s\"\n", testItem1, pList->AddItem((void *)testItem1) ? strTrue : strFalse);
	
	const char *testItem2 = "13";
	printf("  pList->HasItem(\"%s\") == \"%s\"\n", testItem2, pList->HasItem(testItem2) ? strTrue : strFalse);
	printf("  pList->IndexOf(\"%s\") == %ld\n", testItem2, pList->IndexOf(testItem2));
	printf("  pList->AddItem(\"%s\") == \"%s\"\n", testItem2, pList->AddItem((void *)testItem2) ? strTrue : strFalse);
	
	const char *testItem3 = "20";
	printf("  pList->HasItem(\"%s\") == \"%s\"\n", testItem3, pList->HasItem(testItem3) ? strTrue : strFalse);
	printf("  pList->IndexOf(\"%s\") == %ld\n", testItem3, pList->IndexOf(testItem3));
	printf("  pList->AddItem(\"%s\") == \"%s\"\n", testItem3, pList->AddItem((void *)testItem3) ? strTrue : strFalse);

	printf("  All items == { ");
	for (int i=0; i < pList->CountItems()-1; i++)
	{
		printf( "\"%s\", ", static_cast<const char *>(pList->ItemAt(i)));
	}
	printf( "\"%s\" }\n", static_cast<const char *>(pList->ItemAt(pList->CountItems()-1)));

	pList->SortItems(CompareStrAsDecimalValue);
	
	printf("  All items after sorting == { ");
	for (int i=0; i < pList->CountItems()-1; i++)
	{
		printf( "\"%s\", ", static_cast<const char *>(pList->ItemAt(i)));
	}
	printf( "\"%s\" }\n", static_cast<const char *>(pList->ItemAt(pList->CountItems()-1)));

	printf("  pList->CountItems() == %ld\n", pList->CountItems());
	printf("  pList->FirstItem() == \"%s\"\n", (const char *)pList->FirstItem());
	printf("  pList->LastItem() == \"%s\"\n", (const char *)pList->LastItem());
	
	delete pList;
	pList = NULL;
	
	return 0;
}

/**
 *	EOF
 */
